"""Gaussian Process.
"""

import numpy as np

from base import RandomProcess

from typing import Iterable


class GaussianProcess(RandomProcess):
    """Gaussian process.

    A stochastic process :math:`X = (X_t)_{t \\geq 0}` is a
    **Gaussian process** if for all :math:`n \\in \\mathbb{N}` and all
    time sequences :math:`t_1,\\ldots,t_n \\geq 0` with
    :math:`0 < t_1 < \\ldots < t_n` applies

    .. math::

        \\left( X_{t_1}, \\ldots, X_{t_n} \\right) \\sim \\mathcal{N}
        (m, \\Sigma)

    with :math:`m` a vector of expectations and :math:`\\Sigma` a
    covariance matrix, i.e.
    :math:`\\left( X_{t_1}, \\ldots, X_{t_n} \\right)` follows a
    :math:`n`-dimensional normal distribution.

    Notes
    -----
    For more information, see
    `Gaussian process <https://en.wikipedia.org/wiki/Gaussian_process>`_.

    Parameters
    ----------
    t : float, optional
        The right hand side of the time interval :math:`[0,t]`, by
        default 1.0.
    """

    def __init__(self, t: float = 1) -> None:
        super().__init__(t)

    def sample(self, n: int) -> np.array:
        return super().sample(n)

    def sample_at(self, times: Iterable[float]) -> np.array:
        return super().sample_at(times)



"""Utility functions.
"""
from typing import List
from typing import Iterable

import numpy as np


def generate_times(end: float = 1.0, n: int = 10) -> List[float]:
    """Generate times.

    Generate a linspace from 0 to ``end`` for ``n`` increments. That is
    divide the time interval :math:`[0, end]` in :math:`n` equidistant
    intervals.

    Parameters
    ----------
    n : int, optional
        The number of increments, by default 10.
    end : float, optional
        The right hand side of the interval :math:`[0,end]`, by default
        1.0.

    Returns
    -------
    List[float]
        A vector of times.
    """
    return np.linspace(0, end, n+1)


def check_times(times: Iterable[float]) -> None:
    """Check times.

    Check if the times vector is a sequence of strictly increasing and
    nonnegative floats.

    Parameters
    ----------
    times : Iterable[float]
        A vector of times.

    Returns
    -------
    None

    Raises
    ------
    TypeError
        Times are not a vector of floats.
    ValueError
        Times are not nonnegative or not strictly increasing.
    """
    if np.any([not isinstance(t, float)] for t in times):
        raise TypeError('Times must be an iterable of floats.')
    if np.any([t < 0 for t in times]):
        raise ValueError('Times must be nonnegative.')
    increments = np.diff(times)
    if np.any([t < 0 for t in increments]):
        raise ValueError('Times must be strictly increasing.')
    return None



import setuptools

package_name = "stochpro"

# Requirements
requirements_file = open("requirements.txt", 'r')
requirements = [i.strip() for i in requirements_file.readlines()]
test_requirements_file = open("./tests/requirements.txt", 'r')
test_requirements = [i.strip() for i in requirements_file.readlines()]
docs_requirements_file = open("./docs/requirements.txt", 'r')
docs_requirements = [i.strip() for i in requirements_file.readlines()]

# ReadMe
long_description = open("README.rst", "r").read()

url = "https://github.com/dsagolla/stochpro"
download_url = url + "/archive/main.zip"
documentation_url = "https://stochpro.readthedocs.io/en/latest/"
tracker_url = url + "/issues"

# Setup
setuptools.setup(
    name="stochpro",
    version="0.0.1",
    author="Daniel Sagolla",
    author_email="dsagolla@mailbox.org",
    description=r"Sampling from stochastic processes.",
    long_description=long_description,
    long_description_content_type='text/x-rst',
    keywords='probability stochastic random sampling sample simulate ' +
                'simulating stochastic-process process gaussian ' +
                'gaussian-process brownian-motion ito ito-process',
    url=url,
    download_url=download_url,
    project_urls={
        "Documentation": documentation_url,
        "Source": url,
        "Tracker": tracker_url,
    },
    platforms=['Linux', 'OSX', 'Windows'],
    packages=setuptools.find_packages(
        where='src',
        exclude=('tests', 'docs')
    ),
    package_dir={'': 'src'},
    install_requires=requirements,
    python_requires='>=3.7',
    setup_requires=[
        'setuptools',
        'pytest-runner'],
    tests_require=[
        'pytest',
    ],
    include_package_data=True,
    zip_safe=False,
    extras_require={
        'test': test_requirements,
        'docs': docs_requirements,
    },
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
1



"""Base class.
"""
import numpy as np

from typing import Iterable

from abc import ABC
from abc import abstractmethod


class RandomProcess(ABC):
    """Base class for random time processes to be subclassed by all
    processes.

    Parameters
    ----------
    t : float, optional
        The right hand side of the time interval :math:`[0,t)`, by
        default 1.0.

    Methods
    -------
    sample
    sample_at
    """

    rng = np.random.default_rng()

    def __init__(self, t: float = 1.0) -> None:
        self.t = t

    @abstractmethod
    def sample(self, n: int) -> np.array:
        pass

    @abstractmethod
    def sample_at(self, times: Iterable[float]) -> np.array:
        pass
